@@ -20,6 +20,10 @@
  *
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdlib.h> /* getenv */
 #include <string.h> /* memcpy */
 #include <ne_socket.h>
@@ -29,9 +33,16 @@
 #include <ne_utils.h>
 #include <ne_request.h>
 #include <ne_dates.h>
+#include <glib.h>
 
 #include "httpfetch.h"
 
+#ifdef DEBUG
+#define DEBUG_PRINT(...) g_message("httpfetch** " __VA_ARGS__);
+#else
+#define DEBUG_PRINT(...)
+#endif
+
 static void init_proxy(ne_session* ne_sess)
 {
 	char *pchProxy; 
@@ -39,19 +50,19 @@ static void init_proxy(ne_session* ne_sess)
 
  	if((pchProxy = getenv("http_proxy")) != NULL){
 
-		NE_DEBUG(NE_DBG_HTTP, "use proxy: %s", pchProxy);
+		DEBUG_PRINT("use proxy: %s", pchProxy);
 
 		if(ne_uri_parse(pchProxy, &uri) 
 		   || uri.host == NULL){
-			NE_DEBUG(NE_DBG_HTTP, "Coud not parse proxy URI");
+			DEBUG_PRINT("Coud not parse proxy URI");
 			return;
 		}
 		if (uri.scheme == NULL){
-			NE_DEBUG(NE_DBG_HTTP, "not found uri scheme");
+			DEBUG_PRINT("not found uri scheme");
 			uri.scheme = "http";
 		}
 		if(uri.port == 0){
-			NE_DEBUG(NE_DBG_HTTP, "use default port");
+			DEBUG_PRINT("use default port");
 			uri.port = ne_uri_defaultport(uri.scheme);
 		}
 
@@ -75,10 +86,32 @@ buffer_append(void* userdata, const char* buff, size_t len)
 
 	memcpy(&ne_buf->data[ne_buf->used], buff, len);
 	ne_buf->used += len;
+
+	return;
 }
 
-int
+/*
+static void
+buffer_append(void* userdata, const char* buff, size_t len)
+{
+	GString* strBuff = (GString*)userdata;
+	
+	if(len == 0)
+		return;
+
+	g_string_append_len(strBuff, buff, len);
+
+	return;
+	
+}
+*/
+
+/*int
 httpfetch(const char* strURI, char* const buff, int bufflen, 
+time_t tmModified, time_t* ptmRetModified)*/
+
+int
+httpfetch(const char* strURI, char** ppBuff, 
 		  time_t tmModified, time_t* ptmRetModified)
 {
 	ne_session *ne_sess = NULL;
@@ -86,21 +119,23 @@ httpfetch(const char* strURI, char* const buff, int bufflen,
 	ne_buffer* ne_buf = ne_buffer_create();
 	ne_buffer* ne_header_buf = ne_buffer_create();
 	ne_uri uri = {0};
-	int retlen;
+	int retlen = 0;
+
+	*ppBuff = NULL;
 
 	ne_buf->used = 0;
 	ne_header_buf->used = 0;
 
     /* Initialize socket libraries */
     if (ne_sock_init()) {
-        NE_DEBUG(NE_DBG_HTTP, "Failed to initialize socket libraries.");
+        DEBUG_PRINT("Failed to initialize socket libraries.");
 		retlen = HF_ERROR;
 		goto ret;
     }
 
 	if(ne_uri_parse(strURI, &uri) 
 	   || uri.host == NULL || uri.path == NULL){
-		NE_DEBUG(NE_DBG_HTTP, "Coud not parse URI");
+		DEBUG_PRINT("Coud not parse URI");
 		retlen = HF_ERROR;
 		goto ret;
 	}
@@ -127,48 +162,41 @@ httpfetch(const char* strURI, char* const buff, int bufflen,
 	
 	if(tmModified > -1){
 		char* chDate = (char*)ne_rfc1123_date(tmModified);
-		NE_DEBUG(NE_DBG_HTTP, "modified str: %s\n", chDate);
+		DEBUG_PRINT("modified str: %s\n", chDate);
 		ne_add_request_header(ne_req, "If-Modified-Since", chDate);
 		free(chDate);
 	}
 
 	if(NE_OK != ne_request_dispatch(ne_req)){
-		NE_DEBUG(NE_DBG_HTTP, "dispatch error");
+		DEBUG_PRINT("dispatch error");
 		retlen = HF_ERROR;
 		goto ret;
 	}
 
 	if(304 == ne_get_status(ne_req)->code){
-		NE_DEBUG(NE_DBG_HTTP, "return code 304");
+		DEBUG_PRINT("return code 304");
 		retlen = HF_NOT_MODIFIED;
 		goto ret;
 	}
 
 	if(ptmRetModified){
 		*ptmRetModified = ne_rfc1123_parse(ne_header_buf->data);
-		NE_DEBUG(NE_DBG_HTTP, "modified %d\n", (int)*ptmRetModified);
+		DEBUG_PRINT("modified %d\n", (int)*ptmRetModified);
 	}
-	
-	if(ne_buf->used + 1> bufflen){
-		retlen = bufflen;
-	}
-	else{
-		retlen = ne_buf->used + 1;
-	}
-	
-	memcpy(buff, ne_buf->data, retlen - 1);
-	buff[retlen - 1] = '\0';
 
+	if(ne_buf->used > 0){
+		retlen = ne_buf->used + 1;
+		*ppBuff = malloc(retlen);
+		memcpy(*ppBuff, ne_buf->data, retlen - 1);
+		(*ppBuff)[retlen - 1] = '\0';
+	}
+	
  ret:
 	ne_buffer_destroy(ne_buf);
 	ne_buffer_destroy(ne_header_buf);
 	if(ne_req)ne_request_destroy(ne_req);
 	if(ne_sess)ne_session_destroy(ne_sess);
 	ne_sock_exit();
-
+	DEBUG_PRINT("return: %d", retlen);
 	return retlen;
 }
-
-#ifdef HTTPFETCH_MAIN
-
-#endif
