@@ -33,8 +33,14 @@
 #endif
 
 #include <stdio.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+
 #include <libxml/uri.h>
 #include <pthread.h>
+#include <ne_uri.h>
 
 #include "xmlparse.h"
 #include "httpfetch.h"
@@ -106,9 +112,32 @@ create_new_feeddata()
 	return pFeedData;
 }
 
+static feeddata_t*
+copy_feeddata(feeddata_t* pFeedData)
+{
+	feeddata_t* pNewFeedData;
+
+	if(!pFeedData)
+		return NULL;
+
+	pNewFeedData = create_new_feeddata();
+	if(pFeedData->pchURI)
+		pNewFeedData->pchURI = g_strdup(pFeedData->pchURI);
+
+	pNewFeedData->pFeed = copy_feed(pFeedData->pFeed);
+	pNewFeedData->tmLastModified = pFeedData->tmLastModified;
+	pNewFeedData->pIcon = pFeedData->pIcon;
+	pNewFeedData->iValidHours = pFeedData->iValidHours;
+	
+	return pNewFeedData;
+}
+
 void
 free_feeddata(feeddata_t* pFeedData)
 {
+	if(!pFeedData)
+		return;
+
 	if(pFeedData->pFeed){
 		free_feed(pFeedData->pFeed);
 	}
@@ -157,19 +186,18 @@ static void
 free_outlineList(GList* pOutlineList)
 {
 	GList* pList;
-	outline_t* pOutline;
 	
+	DEBUG_PRINT("free_outlineList start");
+
 	for(pList = pOutlineList; pList; pList = pList->next){
-		if(!(pOutline = (outline_t*)pList->data))
-			continue;
-		if(pOutline->strTitle)
-			g_string_free(pOutline->strTitle, TRUE);
-		if(pOutline->strURL)
-			g_string_free(pOutline->strURL, TRUE);
-		g_free(pOutline);
+		free_outline((outline_t*)pList->data);
 	}
 
 	g_list_free(pOutlineList);
+
+	DEBUG_PRINT("free_outlineList end");
+
+	return;
 }
 
 static void
@@ -198,6 +226,118 @@ StoreFeedList2File(newsreader_t* newsreader)
 	return;
 }
 
+static void
+write_cache_file(const gchar* xmlBuff, int bufflen, gchar* pchURI, GHashTable* pTable)
+{
+	ne_uri uri = {0};
+	GString* strFile;
+	gchar* pchFileName;
+	gchar* pchPath;
+	gchar* pPrefix;
+	int fd;
+
+	if(!pchURI){
+		g_warning("URI is NULL!!!");
+		return;
+	}
+	
+	NEWSREADER_LOCK();
+
+	if(!(pchFileName = (gchar*)g_hash_table_lookup(pTable, pchURI))){
+		if(ne_uri_parse(pchURI, &uri)
+		   || uri.host == NULL){
+			g_warning("ne_uri_parse faild");
+			NEWSREADER_UNLOCK();
+			return;
+		}
+		
+		pPrefix = g_strdup(uri.host);
+		pPrefix = g_strdelimit(pPrefix, ".", '_');
+		strFile = g_string_new(pPrefix);
+		g_string_append_printf(strFile, "%08X", (unsigned int)random());
+		pchFileName = g_string_free(strFile, FALSE);
+		g_hash_table_insert(pTable, g_strdup(pchURI), pchFileName);
+	}
+
+	strFile = g_string_new("xfce4" G_DIR_SEPARATOR_S "newsreader" G_DIR_SEPARATOR_S);
+	g_string_append(strFile, pchFileName);
+	pchPath = xfce_resource_save_location(XFCE_RESOURCE_CACHE, strFile->str, TRUE);
+	g_string_free(strFile, TRUE);
+
+	DEBUG_PRINT("write cahce file: %s", pchPath);
+
+	if((fd = open(pchPath, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR)) < 0){
+		g_warning("open faild");
+		g_free(pchPath);
+		NEWSREADER_UNLOCK();
+		return;
+	}
+	
+	if(write(fd, xmlBuff, bufflen - 1) < 0){
+		g_warning("write faild");
+		g_free(pchPath);
+		close(fd);
+		NEWSREADER_UNLOCK();
+		return;
+	}
+
+	g_free(pchPath);
+	close(fd);
+	NEWSREADER_UNLOCK();
+
+	return;
+}
+
+static gchar*
+get_cache_file(gchar* pchURI, int* plen, GHashTable* pTable)
+{
+	gchar* pchFile;
+	GString* strFile;
+	gchar* pchPath;
+	GArray* pData;
+	gchar buff[1024 * 10];
+	int bufflen;
+	int fd;
+	
+	DEBUG_PRINT("get_cache_file start");
+
+	if(!pchURI)
+		return NULL;
+
+	if(!(pchFile = g_hash_table_lookup(pTable, pchURI))){
+		g_warning("not found cache file: %s", pchURI);
+		return NULL;
+	}
+	
+	strFile = g_string_new("xfce4" G_DIR_SEPARATOR_S "newsreader" G_DIR_SEPARATOR_S);	
+	g_string_append(strFile, pchFile);
+	pchPath = xfce_resource_lookup(XFCE_RESOURCE_CACHE, strFile->str);
+	g_string_free(strFile, TRUE);
+	if(!pchPath)
+		return NULL;
+
+	DEBUG_PRINT("Get Cache file: %s", pchPath);
+
+	pData = g_array_new(TRUE, FALSE, sizeof(gchar));
+	if((fd = open(pchPath, O_RDONLY)) < 0){
+		g_warning("open faild");
+		g_free(pchPath);
+		return NULL;
+	}
+
+	while((bufflen = read(fd, buff, 1024*10)) > 0){
+		pData = g_array_append_vals(pData, buff, bufflen);
+	}
+
+	close(fd);
+
+	*plen = pData->len;
+
+	DEBUG_PRINT("get_cache_file end");
+
+	return g_array_free(pData, FALSE);
+}
+
 /* based on gaim_program_is_valid */
 static gboolean
 program_is_valid(const char *program)
@@ -309,7 +449,7 @@ msg_display_func(newsreader_t* newsreader)
 			g_free(pMsgdata);
 		}
 		else{
-			DEBUG_PRINT("??? pMsgdata is null");
+			DEBUG_PRINT("pMsgdata is null");
 		}
 		
 		DEBUG_PRINT(pchMsg);
@@ -560,38 +700,38 @@ UpdateFeedListMenu(newsreader_t* newsreader)
 static feeddata_t*  CreateStaticFeed(void)
 {
 
-  feeddata_t* pFeedData = NULL;
-  feed_t* pFeed = NULL;
-  item_t* pItem;
+	feeddata_t* pFeedData = NULL;
+	feed_t* pFeed = NULL;
+	item_t* pItem;
  
-  pFeedData = create_new_feeddata();
-  pFeedData->pchURI = g_strdup("");
+	pFeedData = create_new_feeddata();
+	pFeedData->pchURI = g_strdup("");
 
-  pFeed = create_new_feed();
-  pFeed->title = g_string_new("newsreader");
-  pFeed->url = g_string_new("");
+	pFeed = create_new_feed();
+	pFeed->title = g_string_new("Newsreader");
+	pFeed->url = g_string_new("");
   
-  pItem = create_new_item();
-  pItem->title = g_string_new("Newsreader");
-  pItem->url = g_string_new("");
+	pItem = create_new_item();
+	pItem->title = g_string_new("Newsreader ");
+	g_string_append(pItem->title, VERSION);
+	pItem->url = g_string_new("");
 
-  pFeed->items = g_list_append(pFeed->items, pItem);
+	pFeed->items = g_list_append(pFeed->items, pItem);
 
-  pFeedData->pFeed = pFeed;
+	pFeedData->pFeed = pFeed;
  
-  return pFeedData;
+	return pFeedData;
 }
 
 static GdkPixbuf*
 httpfetch_favicon(const char* uri)
 {
-
-	gchar buff[MAX_FETCH_LEN];
+	gchar* buff;
 	int bufflen;
 	GdkPixbufLoader* loader;
 	GdkPixbuf* pixbuf = NULL;
 
-	if((bufflen = httpfetch(uri, buff, MAX_FETCH_LEN, -1, NULL)) < 0){
+	if((bufflen = httpfetch(uri, &buff, -1, NULL)) < 0){
 		g_warning("favicon fetch failed");
 		return NULL;
 	}
@@ -602,6 +742,8 @@ httpfetch_favicon(const char* uri)
 	pixbuf = gdk_pixbuf_loader_get_pixbuf(loader);
 	gdk_pixbuf_loader_close(loader, NULL);
 
+	g_free(buff);
+
 	return pixbuf;
 }
 
@@ -683,8 +825,11 @@ get_favicon(feed_t* pfeed)
 static void
 AppendIconList(newsreader_t* newsreader, feeddata_t* pFeedData)
 {
+	GList* poList = NULL;
+	GdkPixbuf* pPixbuf;
 
-	GList* poList = newsreader->poIconList;
+	NEWSREADER_LOCK();
+	poList = newsreader->poIconList;
 	favicon_t* favicon = NULL;
 	/* List have target icon? */
 	for(; poList; poList = poList->next){
@@ -695,15 +840,19 @@ AppendIconList(newsreader_t* newsreader, feeddata_t* pFeedData)
 			break;
 		}
 	}
+	NEWSREADER_UNLOCK();
 
 	if(!poList){
 		favicon = g_new0(favicon_t, 1);
 		favicon->uri = g_strdup(pFeedData->pFeed->url->str);
-		favicon->pIconPixbuf = get_favicon(pFeedData->pFeed);
-		if(favicon->pIconPixbuf){
-			favicon->pIconPixbuf = gdk_pixbuf_scale_simple(favicon->pIconPixbuf, 16, 16, GDK_INTERP_BILINEAR);
+		pPixbuf = get_favicon(pFeedData->pFeed);
+		if(pPixbuf){
+			favicon->pIconPixbuf = gdk_pixbuf_scale_simple(pPixbuf, 16, 16, GDK_INTERP_BILINEAR);
+			g_free(pPixbuf);
 		}
+		NEWSREADER_LOCK();
 		newsreader->poIconList = g_list_append(newsreader->poIconList, favicon);
+		NEWSREADER_UNLOCK();
 	}
 
 	return;
@@ -711,39 +860,79 @@ AppendIconList(newsreader_t* newsreader, feeddata_t* pFeedData)
 
 static feed_t* 
 GetFeed(char* buff, int len)
-{
+{  
+	xmlDoc *doc = NULL;
+	xmlNode *root = NULL;
+	feed_t* pfeed;
   
-  xmlDoc *doc = NULL;
-  xmlNode *root = NULL;
-  feed_t* pfeed;
+	if(!(doc = xmlParseMemory(buff, len))){
+		g_warning("!!! parse error");
+		return NULL;
+	}
   
-  if(!(doc = xmlParseMemory(buff, len))){
-	g_warning("!!! parse error");
-	return NULL;
-  }
+	if(!(root = xmlDocGetRootElement(doc))){
+		g_warning("!!! get root element");
+		xmlFreeDoc(doc);
+		return NULL;
+	}
   
-  if(!(root = xmlDocGetRootElement(doc))){
-	g_warning("!!! get root element");
+	if(!(pfeed = get_feed(doc, root))){
+		g_warning("!!! get feed");
+	}
+
+	xmlUnlinkNode(root);
+	xmlFreeNode(root);
 	xmlFreeDoc(doc);
-	return NULL;
-  }
-  
-  if(!(pfeed = get_feed(doc, root))){
-	  g_warning("!!! get feed");
-  }
-
-  xmlUnlinkNode(root);
-  xmlFreeNode(root);
-  xmlFreeDoc(doc);
-
-  return pfeed;
 
+	return pfeed;
 }
 
+/* tmOldModified$B$h$j?7$7$$5-;v$N(BbIsNew$B$r(BTRUE$B$K(B */
+static void
+set_new_flag(feeddata_t* pFeedData, time_t tmOldModified)
+{
+	GList *pList;
+	item_t *pItem;
+
+	if(!pFeedData || !pFeedData->pFeed)
+		return;
+
+	if(tmOldModified < 0)
+		return;
+
+	for(pList = pFeedData->pFeed->items; pList; pList = pList->next){
+		if(!(pItem = (item_t*)pList->data))
+			continue;
+		if( pItem->tmModified > tmOldModified)
+			pItem->bIsNew = TRUE;
+	}
+	
+	return;
+}
+
+static void
+set_new_flag_all(feeddata_t* pFeedData, gboolean bIsNew)
+{
+	GList *pList;
+	item_t *pItem;
+
+	if(!pFeedData || !pFeedData->pFeed)
+		return;
+
+	for(pList = pFeedData->pFeed->items; pList; pList = pList->next){
+		if(!(pItem = (item_t*)pList->data))
+			continue;
+		pItem->bIsNew = bIsNew;
+	}
+	
+	return;
+}
+
+/* $B%U%#!<%I%j%9%H$h$j%U%#!<%I%G!<%?$r<hF@$9$k(B */
 static gpointer
 GetFeedList(newsreader_t* newsreader)
 {
-	char buff[MAX_FETCH_LEN];
+	char* buff;
 	int feedNo = 0;
 	int bufflen;
 	feed_t* pfeed;
@@ -757,12 +946,10 @@ GetFeedList(newsreader_t* newsreader)
 	GList* poNewList = NULL;
 	GList* poOldList = NULL;
 	GList* pOutlineList = NULL;
-	/*static GStaticMutex newsreader_feedlist_mutex = G_STATIC_MUTEX_INIT;*/
 
-	/*	if(!(g_static_mutex_trylock (&newsreader_feedlist_mutex)))
-		return NULL;*/
 	if(pthread_mutex_trylock(&getfeedlist_mutex)){
-		DEBUG_PRINT("GetFeedList Locked");
+		/* $B$9$G$K(BGetFeedList$B$,<B9TCf(B */
+		DEBUG_PRINT("GetFeedList have started");
 		return NULL;
 	}
 	
@@ -771,8 +958,9 @@ GetFeedList(newsreader_t* newsreader)
 	pthread_mutex_lock(&getfeed_mutex);
 
 	NEWSREADER_LOCK();
-	DEBUG_PRINT("copy_outlineList");
+	DEBUG_PRINT("copy_outlineList start");
 	pOutlineList = copy_outlineList(newsreader);
+	DEBUG_PRINT("copy_outlineList end");
 	NEWSREADER_UNLOCK();
 
 	poNewList = g_list_append(poNewList, CreateStaticFeed());
@@ -781,6 +969,7 @@ GetFeedList(newsreader_t* newsreader)
 	outline_t* poOutline;
 
 	for(plist = pOutlineList; plist; plist = plist->next){
+		pFeedData = NULL;
 		poOutline = (outline_t*)plist->data;
 		if(!poOutline || !poOutline->bAvailable)
 			continue;
@@ -792,35 +981,59 @@ GetFeedList(newsreader_t* newsreader)
 			}
 		}
 		
+		DEBUG_PRINT("search feedlist end");
+
+		/* $BA02s<hF@$N(BFeedData$B$,$"$C$?(B */
 		if(poFeedList){
-			pFeedData = (feeddata_t*)poFeedList->data;
-		
-			if((bufflen = httpfetch(pFeedData->pchURI, buff, MAX_FETCH_LEN, pFeedData->tmLastModified, &tmRetLastModified))){
+			pFeedData = copy_feeddata((feeddata_t*)poFeedList->data);
+			if((bufflen = httpfetch(pFeedData->pchURI, &buff, pFeedData->tmLastModified, &tmRetLastModified)) < 0){
+				DEBUG_PRINT("httpfetch faild");
 				if(HF_NOT_MODIFIED == bufflen ){
-					DEBUG_PRINT(" newsreader - feed not modified");
+					DEBUG_PRINT("feed not modified");
 					pFeedData->iValidHours = poOutline->iValidHours;
+					set_new_flag_all(pFeedData, FALSE);
 					poNewList = g_list_append(poNewList, pFeedData);
 					feedNo++;
 					continue;
 				}
-				else if(HF_ERROR == bufflen){
-					g_warning("!!! newsreader - httpfetch error for exits feed");
-					continue;
+
+				if(HF_ERROR == bufflen){
+					DEBUG_PRINT("HF_ERROR");
+					if(!(buff = get_cache_file(pFeedData->pchURI, &bufflen, newsreader->pCacheTable))){
+						free_feeddata(pFeedData);
+						g_warning("httpfetch error for exits feed");
+						continue;
+					}
 				}
-				
 			}
+			else{
+				/* store buff to cache file */
+				write_cache_file(buff, bufflen, poOutline->strURL->str, newsreader->pCacheTable);
+			}
+			
 		}
 		else{
-			if((bufflen = httpfetch(poOutline->strURL->str, buff, MAX_FETCH_LEN, -1, &tmRetLastModified)) < 0){
-				g_warning("!!! newsreader - httpfetch error");
-				continue;
+			if((bufflen = httpfetch(poOutline->strURL->str, &buff, -1, &tmRetLastModified)) < 0){
+				DEBUG_PRINT("httpfetch faild");
+				if(!(buff = get_cache_file(poOutline->strURL->str, &bufflen, newsreader->pCacheTable))){
+					DEBUG_PRINT("get_cache_file faild");
+					continue;
+				}
 			}
+			else{
+				write_cache_file(buff, bufflen, poOutline->strURL->str, newsreader->pCacheTable);
+			}
+			
 		}
 
 		if(!(pfeed = GetFeed(buff, bufflen))){
-			g_warning("!!! newsreader - GetFeed error");
+			free_feeddata(pFeedData);
+			g_warning("GetFeed error");
 			continue;
 		}
+
+		if(buff)
+			g_free(buff);
 		
 		pNewFeedData = create_new_feeddata();
 		pNewFeedData->pchURI = g_strdup(poOutline->strURL->str);
@@ -828,6 +1041,17 @@ GetFeedList(newsreader_t* newsreader)
 		pNewFeedData->tmLastModified = tmRetLastModified;
 		pNewFeedData->iValidHours = poOutline->iValidHours;
 
+		if(pFeedData){
+			/* $B8=>u(Bfeed_t$B$N(BtmModified$B%U%#!<%k%I$O<hF@$7$F$$$J$$$N$G(B
+			   feeddata_t$B$N(BtmLastModified$B$r;HMQ$7$F$*$/(B*/
+			set_new_flag(pNewFeedData, pFeedData->tmLastModified);
+		}
+		else{
+			set_new_flag_all(pNewFeedData, FALSE);
+		}
+
+		free_feeddata(pFeedData);
+
 		DEBUG_PRINT("newsreader get URI: %s", pNewFeedData->pchURI);
 		DEBUG_PRINT("newsreader - modified %d", (int)tmRetLastModified);
 
@@ -848,24 +1072,14 @@ GetFeedList(newsreader_t* newsreader)
 	newsreader->g_pFeedList = poNewList;
 
 	for(poFeedList = poOldList; poFeedList; poFeedList = poFeedList->next){
-		for(plist = newsreader->g_pFeedList; plist; plist = plist->next){
-			if(poFeedList->data != NULL &&
-			   plist->data == poFeedList->data)
-				break;
-		}
-		if(plist)
-			continue;
-		/* free old feeddata */	
 		free_feeddata((feeddata_t*)poFeedList->data);
 	}
-	
 	g_list_free(poOldList);
 
 	UpdateFeedListMenu(newsreader);
 
 	NEWSREADER_UNLOCK();
 
-	/*	g_static_mutex_unlock (&newsreader_feedlist_mutex);*/
 	pthread_mutex_unlock(&getfeed_mutex);
 	pthread_mutex_unlock(&getfeedlist_mutex);
 
@@ -943,8 +1157,7 @@ newsreader_item_next(newsreader_t* newsreader, time_t tmCur)
 /* current display item pointer to prev */
 static void
 newsreader_item_prev(newsreader_t* newsreader, time_t tmCur)
-{
-	
+{	
 	gint iValidHours;
 	GList* pList;
 
@@ -1004,8 +1217,8 @@ display_current_item(newsreader_t* newsreader)
 	if(pItem->tmModified >= 0){
 		t_st = localtime(&pItem->tmModified);
 		g_string_printf(strDate, "%04d/%02d/%02d %02d:%02d", 
-				 t_st->tm_year+1900, t_st->tm_mon+1, t_st->tm_mday,
-				 t_st->tm_hour, t_st->tm_min);
+						t_st->tm_year+1900, t_st->tm_mon+1, t_st->tm_mday,
+						t_st->tm_hour, t_st->tm_min);
 		g_string_append (strLabel, " - ");
 		g_string_append (strLabel, strDate->str);
 	}
@@ -1251,6 +1464,7 @@ newsreader_attach_callback (Control * control, const char *signal,
     newsreader_t *newsreader = control->data;
 
     g_signal_connect (newsreader->widget, signal, callback, data);
+    g_signal_connect (newsreader->wBtnMenu, signal, callback, data);
 }
 
 static gboolean
@@ -1333,7 +1547,6 @@ on_enter_notify_cb (GtkEventBox *widget, GdkEventCrossing *event, newsreader_t*
 	if(newsreader->bUseTooltips){
 		gtk_tooltips_enable(newsreader->wTooltips);
 	}
-	
 
 	return TRUE;  
 }
@@ -1387,6 +1600,33 @@ on_button_press_cb(GtkEventBox *widget, GdkEventKey *event, newsreader_t* newsre
 	return FALSE;
 }
 
+static void
+clear_cache_dir()
+{
+	
+	gchar** ppchFileMatch;
+	gchar** ppTemp;
+	gchar* pchFileName;
+
+	ppchFileMatch = xfce_resource_match(XFCE_RESOURCE_CACHE, 
+								   "xfce4" G_DIR_SEPARATOR_S "newsreader" G_DIR_SEPARATOR_S "*",
+								   FALSE);
+
+	for(ppTemp = ppchFileMatch; *ppTemp; *ppTemp++){
+		pchFileName = xfce_resource_lookup(XFCE_RESOURCE_CACHE, *ppTemp);
+		DEBUG_PRINT("unlink: %s", pchFileName);
+		if(unlink(pchFileName) < 0){
+			g_warning(strerror(errno));	
+		}
+		
+		g_free(pchFileName);
+	}
+	
+	g_strfreev(ppchFileMatch);
+
+	return;
+}
+
 /**
  * newsreader_new
  *
@@ -1403,8 +1643,12 @@ newsreader_new (void)
 	GtkWidget *image1;
 	GtkWidget *eventbox1;
 
+	srandom((int)time(NULL));
+
     newsreader = g_new0 (newsreader_t, 1);
 
+	clear_cache_dir();
+
 	/*	pthread_create(&newsreader->msgdisp_tid, NULL, (void*)msg_display_func, (void *) newsreader);*/
 
     /* initialize default settings */
@@ -1414,6 +1658,8 @@ newsreader_new (void)
 
 	newsreader->pchBrowserCmd = g_strdup("firefox %s");
 
+	newsreader->pCacheTable = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
+
     /* this is our main widget that catches all events */
 	hbox1 = gtk_hbox_new(FALSE, 0);
     newsreader->widget = hbox1;
@@ -1454,14 +1700,11 @@ newsreader_new (void)
 	/* create model data*/
 	newsreader->oGUI.wModel =GTK_TREE_MODEL( gtk_list_store_new (NUM_COLS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT));	
 
-
 	/* get feed-list from file */
 	gchar* filename;
 	xmlDocPtr doc = NULL;
 	filename = xfce_resource_lookup (XFCE_RESOURCE_CONFIG,
-                                     "xfce4" G_DIR_SEPARATOR_S "newsreader.xml");
-	if (!filename)
-        filename = xfce_get_userfile("newsreader.xml", NULL);
+                                     "xfce4" G_DIR_SEPARATOR_S "newsreader.xml");	
 
     if (g_file_test(filename, G_FILE_TEST_EXISTS)) {
 		/* parse the file */
@@ -1524,6 +1767,8 @@ newsreader_free (Control * control)
 	
 	g_object_unref(newsreader->oGUI.wModel);
 
+	g_hash_table_destroy(newsreader->pCacheTable);
+
 	/* don't forget to free the memory for the newsreader_t struct itself */
     g_free (newsreader);
 }
@@ -1647,8 +1892,10 @@ newsreader_apply_options (newsreader_t * newsreader)
 
 		newsreader->poOutlineList = g_list_append(newsreader->poOutlineList, pfeed);
 
-		g_free(iter_title);
-		g_free(iter_url);
+		if(iter_title)
+			g_free(iter_title);
+		if(iter_url)
+			g_free(iter_url);
 
 		valid = gtk_tree_model_iter_next(newsreader->oGUI.wModel, &iter);
 	}
@@ -1919,7 +2166,7 @@ create_initial_menu(newsreader_t* newsreader)
 static void 
 dragedfeed_append_to_list(newsreader_t* newsreader)
 {
-	char buff[MAX_FETCH_LEN];
+	char *buff;
 	gint bufflen;
 	time_t tmLastModified;
 	feed_t* pFeed = NULL;
@@ -1933,7 +2180,7 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 		goto error;
 	}
 
-	if((bufflen = httpfetch(newsreader->pchURI, buff, MAX_FETCH_LEN, -1, &tmLastModified)) < 0){
+	if((bufflen = httpfetch(newsreader->pchURI, &buff, -1, &tmLastModified)) < 0){
 		g_warning("!!! newsreader - httpfetch error");
 		goto error;
 	}
@@ -1943,6 +2190,9 @@ dragedfeed_append_to_list(newsreader_t* newsreader)
 		goto error;
 	}
 
+	if(buff)
+		g_free(buff);
+
 	pFeedData = create_new_feeddata();
 	pFeedData->pchURI = g_strdup(newsreader->pchURI);
 	pFeedData->tmLastModified = tmLastModified;
