@@ -86,13 +86,81 @@ item_t* create_new_item()
 	item_t* pItem = NULL;
 	pItem = g_new0(item_t, 1);
 	pItem->tmModified = -1;
+	pItem->bIsNew = FALSE;
 
 	return pItem;
 }
 
+GList*
+copy_items(GList* pItemList)
+{
+	item_t* pNewItem;
+	item_t* pItem;
+	GList* pNewList = NULL;
+	GList* pList;
+	
+	if(!pItemList)
+		return NULL;
+
+	for(pList = pItemList; pList; pList = pList->next){
+		pItem = (item_t*)pList->data;
+		if(!pItem)
+			continue;
+		pNewItem = create_new_item();
+		if(pItem->title)
+			pNewItem->title = g_string_new(pItem->title->str);
+		if(pItem->url)
+			pNewItem->url = g_string_new(pItem->url->str);
+		pNewItem->tmModified = pItem->tmModified;
+		pNewItem->bIsNew = pItem->bIsNew;
+
+		pNewList = g_list_append(pNewList, pNewItem);
+	}	
+	return pNewList;
+}
+
+feed_t*
+copy_feed(feed_t* pFeed)
+{
+	feed_t* pNewFeed = NULL;
+
+	if(!pFeed)
+		return NULL;
+
+	pNewFeed = create_new_feed();
+	if(pFeed->title)
+		pNewFeed->title = g_string_new(pFeed->title->str);
+	if(pFeed->url)
+		pNewFeed->url = g_string_new(pFeed->url->str);
+	pNewFeed->tmModified = pFeed->tmModified;
+	pNewFeed->items = copy_items(pFeed->items);
+
+	return pNewFeed;
+}
+
+void
+free_outline(outline_t* pOutline)
+{
+	if(!pOutline)
+		return;
+
+	if(pOutline->strTitle)
+		g_string_free(pOutline->strTitle, TRUE);
+
+	if(pOutline->strURL)
+		g_string_free(pOutline->strURL, TRUE);
+
+	g_free(pOutline);
+
+	return;
+}
+
 static void
 free_item(item_t* pitem)
 { 
+	if(!pitem)
+		return;
+
 	if(pitem->title)
 		g_string_free(pitem->title, TRUE);
 
@@ -106,6 +174,10 @@ void
 free_feed(feed_t* pfeed)
 { 
 	GList* plist = NULL;
+
+	if(!pfeed)
+		return;
+
 	for(plist = pfeed->items; plist; plist = plist->next)
 		free_item(plist->data);
 
@@ -196,8 +268,7 @@ get_atom_feed(xmlDocPtr doc, xmlNode* node)
 				return NULL;
 			}
 		}
-	}
-	
+	}	
 	return pFeed;
 }
 
@@ -221,9 +292,7 @@ get_channel(feed_t* pfeed, xmlDocPtr doc, xmlNode* node)
 			pfeed->url = g_string_new(xmlNodeListGetString(doc, cur->children, 1));
 		}
 	}
-
-	return TRUE;
-  
+	return TRUE;  
 }
 
 static gboolean
@@ -254,8 +323,7 @@ get_rss10_item(feed_t* pfeed, xmlDocPtr doc, xmlNode* node)
 	}
 	else{
 		pfeed->items = g_list_append(pfeed->items, pitem);
-	}
-	
+	}	
 	return TRUE;
 }
 
@@ -347,7 +415,6 @@ get_rss20_feed(xmlDocPtr doc, xmlNode* node)
 	return pFeed;
 }
 
-
 feed_t*
 get_feed(xmlDocPtr doc, xmlNode * node)
 {
@@ -490,6 +557,7 @@ expandFolder(xmlNodePtr node, GList* poList)
 	return poList;
 }
 
+/* get outline_t list from XBEL file */
 GList*
 getFeedList_XBEL(xmlDocPtr doc, GList* poList)
 {
@@ -523,6 +591,7 @@ getFeedList_XBEL(xmlDocPtr doc, GList* poList)
 	return poList;
 }
 
+/* store outline_t list to XBEL file*/
 gboolean
 storeFeedList_XBEL(GList* poBookmarkList, gchar* filename)
 {
@@ -569,7 +638,7 @@ storeFeedList_XBEL(GList* poBookmarkList, gchar* filename)
 
 		g_snprintf (value, 4, "%d", poOutline->iValidHours);
 		xmlSetProp(metadata, "ValidHours", value);
-		
+
 		iListNo++;
 	}
 	
@@ -586,6 +655,7 @@ storeFeedList_XBEL(GList* poBookmarkList, gchar* filename)
 
 }
 
+/* store outline_t list to OPML file */ 
 gboolean
 storeOutlineList(GList* poOutlineList, gchar* filename)
 {
